// Copyright 2007, Massachusetts Institute of Technology.
// The use of this code is permitted for research only. There is
// absolutely no warranty for this software.
//
// Author: John Lee (jjl@mit.edu)
//
#ifndef HISTOGRAMS_BIN_H
#define HISTOGRAMS_BIN_H

#include "util/sparse-vector.h"

namespace libpmk {
/// Representation of a histogram bin with a size and a count.
/**
 * A Bin has an identifier of type LargeIndex (a vector of ints). It
 * is also possible to create linked lists or tree structures; each
 * Bin can store pointers to other Bins. However, Bin does not manage
 * these pointers at all, and as such, makes no guarantees about
 * them. For example, the "sibling" pointer can point to an arbitrary
 * bin which may or may not be the real sibling in whatever
 * representation you're using, but it is called "sibling" just for
 * ease of use. This is because MultiResolutionHistogram uses Bins
 * with these kinds of relations.
 */
class Bin {
 public:
   /// Create a new bin identified by a copy of <index>.
   Bin(const LargeIndex& index);
   
   /// Get the LargeIndex identifier.
   const LargeIndex& GetIndex() const;

   /// Sets the size of the bin; returns the old size.
   double SetSize(double size);

   /// Get the size of the bin.
   double GetSize() const;

   /// Increase the count by 1. Returns the final count.
   double IncrementCount();

   /// Decrease the count by 1. Returns the final count.
   double DecrementCount();

   /// Set the count. Returns the new count.
   double SetCount(double count);

   /// Get the current count.
   double GetCount() const;

   Bin* GetParent();
   Bin* GetNextSibling();
   Bin* GetFirstChild();
   void SetParent(Bin* parent);
   void SetNextSibling(Bin* next_sibling);
   void SetFirstChild(Bin* first_child);
   bool HasParent() const;
   bool HasNextSibling() const;
   bool HasChild() const;

   /// Returns true if this bin's index is smaller than that of <other>.
   bool operator<(const Bin& other) const;

   /// Returns true if <one>'s index is smaller than that of <two>.
   static bool CompareBins(const Bin* one, const Bin* two);

 private:
   LargeIndex index_;
   double size_;
   double count_;
   Bin* parent_;
   Bin* next_sibling_;
   Bin* first_child_;
};
}  // namespace libpmk
#endif  // HISTOGRAMS_BIN_H
