// Copyright 2007, Massachusetts Institute of Technology.
// The use of this code is permitted for research only. There is
// absolutely no warranty for this software.
//
// Author: John Lee (jjl@mit.edu)
//
// All siblings are sorted by index.

#include <assert.h>
#include <string>
#include <iostream>
#include "histograms/multi-resolution-histogram.h"
#include "util/sparse-vector.h"

using namespace std;

namespace libpmk {

MultiResolutionHistogram::MultiResolutionHistogram() : num_bins_(1) { 
   LargeIndex empty_index;
   root_bin_ = new Bin(empty_index);
}

MultiResolutionHistogram::~MultiResolutionHistogram() {
   FreeAllBins();
   delete root_bin_;
}

int MultiResolutionHistogram::GetNumBins() const {
   return num_bins_;
}

Bin* MultiResolutionHistogram::GetRootBin() {
   return root_bin_;
}

Bin* const MultiResolutionHistogram::GetRootBin() const {
   return root_bin_;
}

Bin* MultiResolutionHistogram::GetBin(LargeIndex index) {
   return GetBin(index, GetRootBin());
}

Bin* MultiResolutionHistogram::GetBin(LargeIndex index, Bin* finger) {
   LargeIndex desired_index;
   Bin* current_bin = finger;
   LargeIndex finger_index = finger->GetIndex();

   // finger's index should be a prefix of index.
   if (index.size() < finger_index.size()) {
      return NULL;
   }
   for (int ii = 0; ii < (int)finger_index.size(); ++ii) {
      if (index[ii] != finger_index[ii]) {
         return NULL;
      }
      desired_index.push_back(finger_index[ii]);
   }

   for (int current_level = finger_index.size(); ; current_level++) {
      // This means we're at the bin we want
      if (current_level == (int)index.size()) {
         return current_bin;
      }

      // Otherwise, search the next level for the appropriate bin,
      // and set current_bin to that.
      desired_index.push_back(index[current_level]);

      bool found = false;
      for (Bin* next_bin = current_bin->GetFirstChild();
           next_bin != NULL; next_bin = next_bin->GetNextSibling()) {
         if (next_bin->GetIndex().back() == desired_index.back()) {
            current_bin = next_bin;
            found = true;
            break;
         }
      }

      // Quit early if we didn't find that a parent for this index
      // does not exist (and therefore that bin does not exist)
      if (!found) {
         return NULL;
      }
   }

   // This should never happen.
   assert(false);
   return NULL;
}

// The output format is a depth-first traversal of the Bins, including
// the root bin.The depth-first traversal actually runs
// backwards. That is, given the tree
/*
          A
         / \  
        B   C
          / | \
         D  E  F
*/ 
// the order that the bins will be written is A-C-F-E-D-B.
//
// Header: an int32_t saying the total # of bins
// Bin format:
//  int32_t: index_size
//  (index_size * int32_t): the index
//  double: size
//  double: count
void MultiResolutionHistogram::
WriteSingleHistogramToStream(ostream& output_stream,
                             const MultiResolutionHistogram* h) {
   assert(output_stream.good());
   int32_t num_bins = h->GetNumBins();
   output_stream.write((char *)&num_bins, sizeof(int32_t));

   vector<Bin *> todo;
   todo.push_back(h->root_bin_);
   
   while (!todo.empty()) {
      // Process the bin at the top of the stack.
      Bin* current = todo.back();
      LargeIndex current_index = current->GetIndex();
      int32_t index_size = current_index.size();

      double size = current->GetSize();
      double count = current->GetCount();


      output_stream.write((char *)&index_size, sizeof(int32_t));
      for (int ii = 0; ii < index_size; ++ii) {
         int32_t element = current_index[ii];
         output_stream.write((char *)&element, sizeof(int32_t));
      }
      output_stream.write((char *)&size, sizeof(double));
      output_stream.write((char *)&count, sizeof(double));

      // Pop the stack.
      todo.pop_back();

      // Push the bin's children to the top of the stack, if it
      // has any children.
      if (current->HasChild()) {
         Bin* child = current->GetFirstChild();
         while (child != NULL) {
            todo.push_back(child);
            child = child->GetNextSibling();
         }
      }
   }
}


// Depth-first bin deletion.
void MultiResolutionHistogram::FreeAllBins() {
   vector<Bin *> todo;
   todo.push_back(GetRootBin());

   while (!todo.empty()) {
      Bin* current = todo.back();
      if (current->HasChild()) {
         todo.push_back(current->GetFirstChild());
      } else {
         todo.pop_back();
         DeleteLeafBin(current);
      }
   }
   LargeIndex empty_index;
   root_bin_ = new Bin(empty_index);
   num_bins_ = 1;
}

// Cannot delete the root bin.
void MultiResolutionHistogram::DeleteLeafBin(Bin *bin) {
   assert(!bin->HasChild());

   // If it's the root, just delete and quit.
   if (!bin->HasParent()) {
      assert(bin == root_bin_);
      delete bin;
      --num_bins_;
      assert(num_bins_ == 0);
      return;
   }

   if (bin->GetParent()->GetFirstChild() == bin) {
      // If I was the first child, tell my parent to reassign the first
      // child to my next sibling.
      bin->GetParent()->SetFirstChild(bin->GetNextSibling());
   } else {
      // If I was not the first child, I have to tell my previous sibling
      // that its next sibling is my next sibling, not me.
      // We assume that there exists a child of this parent whose
      // NextSibling is me; if this is not satisfied, we abort.
      Bin* iter = bin->GetParent()->GetFirstChild();
      assert(iter != NULL);
      
      while (iter->GetNextSibling() != bin) {
         iter = iter->GetNextSibling();
         assert(iter != NULL);
      }
      
      // iter is now my previous sibling.
      iter->SetNextSibling(bin->GetNextSibling());
   }

   // I am now a dangling node, ready to be deleted.
   delete bin;
   --num_bins_;
}

// Automatically increments num_bins_.
Bin* MultiResolutionHistogram::AddBin(const Bin& new_bin) {
   // Special case: if new_bin is the root bin.
   if (new_bin.GetIndex().size() == 0) {
      double max_size = root_bin_->GetSize() > new_bin.GetSize() ?
         root_bin_->GetSize() : new_bin.GetSize();
      root_bin_->SetSize(max_size);
      root_bin_->SetCount(root_bin_->GetCount() + new_bin.GetCount());
      return root_bin_;
   }
   
   // Try to find the parent bin.
   LargeIndex parent_index = new_bin.GetIndex();
   parent_index.pop_back();
   Bin* parent = GetBin(parent_index);
   assert(parent != NULL);   
   return AddBin(new_bin, parent);
}

Bin* MultiResolutionHistogram::AddBin(const Bin& new_bin, Bin* parent) {
   LargeIndex new_index = new_bin.GetIndex();
   new_index.pop_back();
   if (new_index != parent->GetIndex()) {
      return NULL;
   }

   // Try to see if a bin with this index already exists.
   bool found = false;
   Bin* child = parent->GetFirstChild();
   while (child != NULL) {
      // For equality testing of indices, we only need to check the last
      // element, because we are guaranteed that the parent node is
      // a prefix of the new bin.
      if (new_bin.GetIndex().back() == child->GetIndex().back()) {
         found = true;
         break;
      }
      child = child->GetNextSibling();
   }

   if (found) {
      // If such a bin exists, add the counts, and keep the larger size.
      child->SetCount(child->GetCount() + new_bin.GetCount());
      double max_size = child->GetSize() > new_bin.GetSize() ?
         child->GetSize() : new_bin.GetSize();
      child->SetSize(max_size);
      return child;
   } else {
      // Otherwise, create a copy of new_bin and update the link structure.
      Bin* added = new Bin(new_bin.GetIndex());
      added->SetSize(new_bin.GetSize());
      added->SetCount(new_bin.GetCount());
      added->SetParent(parent);

      if (parent->GetFirstChild() == NULL) {
         // Case 1: the parent was a leaf
         parent->SetFirstChild(added);
         ++num_bins_;
         return added;
      } else {
         Bin* iter = parent->GetFirstChild();
         if (iter->GetIndex().back() > added->GetIndex().back()) {
            // Case 2: added bin is the new first child
            parent->SetFirstChild(added);
            added->SetNextSibling(iter);
            ++num_bins_;
            return added;
         }
         while (iter->HasNextSibling()) {
            // Case 3: added bin is somewhere in the middle
            Bin* next = iter->GetNextSibling();
            if (next->GetIndex().back() > added->GetIndex().back()) {
               iter->SetNextSibling(added);
               added->SetNextSibling(next);
               ++num_bins_;
               return added;
            }
            iter = iter->GetNextSibling();
         }
         // Case 4: added bin is at the end
         iter->SetNextSibling(added);
         ++num_bins_;
         return added;
      }
   }
}

MultiResolutionHistogram* MultiResolutionHistogram::
ReadSingleHistogramFromStream(istream& input_stream) {
   assert(input_stream.good());
   MultiResolutionHistogram* result = new MultiResolutionHistogram();

   int32_t num_bins = 0;
   int32_t zero;
   double root_size;
   double root_count;
   
   input_stream.read((char *)&num_bins, sizeof(int32_t));

   // Special case: the first bin is always the root.
   input_stream.read((char *)&zero, sizeof(int32_t));
   assert(zero == 0);

   input_stream.read((char *)&root_size, sizeof(double));
   input_stream.read((char *)&root_count, sizeof(double));
   result->root_bin_->SetSize(root_size);
   result->root_bin_->SetCount(root_count);

   // (num_bins - 1) since we already read the root bin.
   for (int ii = 0; ii < num_bins - 1; ++ii) {
      int32_t index_size;
      LargeIndex index;
      double size;
      double count;
 
      input_stream.read((char *)&index_size, sizeof(int32_t));
      for (int jj = 0; jj < index_size; ++jj) {
         int32_t index_element;
         input_stream.read((char *)&index_element, sizeof(int32_t));
         index.push_back(index_element);
      }

      input_stream.read((char *)&size, sizeof(double));
      input_stream.read((char *)&count, sizeof(double));
      Bin temp(index);
      temp.SetSize(size);
      temp.SetCount(count);
      result->AddBin(temp);
   }
   return result;
}


void MultiResolutionHistogram::
IgnoreSingleHistogramFromStream(istream& input_stream) {
   assert(input_stream.good());
   int32_t num_bins = 0;
   int32_t zero;   
   input_stream.read((char *)&num_bins, sizeof(int32_t));

   // Special case: the first bin is always the root.
   input_stream.read((char *)&zero, sizeof(int32_t));
   assert(zero == 0);
   
   // Root bin's size and count
   input_stream.ignore(2 * sizeof(double));

   // (num_bins - 1) since we already read the root bin.
   for (int ii = 0; ii < num_bins - 1; ++ii) {
      int32_t index_size;
      input_stream.read((char *)&index_size, sizeof(int32_t));

      // The LargeIndex
      input_stream.ignore(index_size * sizeof(int32_t));

      // The size and the count
      input_stream.ignore(2 * sizeof(double));
   }
}



vector<MultiResolutionHistogram*>
MultiResolutionHistogram::ReadFromStream(istream& input_stream) {
   assert(input_stream.good());
   vector<MultiResolutionHistogram*> histograms;

   int32_t num_hists;
   input_stream.read((char *)&num_hists, sizeof(int32_t));

   for (int ii = 0; ii < num_hists; ++ii) {
      MultiResolutionHistogram* next =
         ReadSingleHistogramFromStream(input_stream);
      histograms.push_back(next);
   }
   return histograms;
}

vector<MultiResolutionHistogram*>
MultiResolutionHistogram::ReadSelectionFromStream(istream& input_stream,
                                                  int start,
                                                  int selection_size) {
   assert(input_stream.good());
   vector<MultiResolutionHistogram*> histograms;
   int32_t num_hists;
   input_stream.read((char *)&num_hists, sizeof(int32_t));

   int end = start + selection_size - 1;
   assert(start >= 0);
   assert(start < num_hists);
   assert(end < num_hists);
   assert(start <= end);
   
   for (int ii = 0; ii < num_hists; ++ii) {
      if (ii >= start && ii <= end) {
         MultiResolutionHistogram* next =
            ReadSingleHistogramFromStream(input_stream);
         histograms.push_back(next);
      } else {
         IgnoreSingleHistogramFromStream(input_stream);
      }
   }
   return histograms;
}



void MultiResolutionHistogram::
WriteToStream(ostream& output_stream,
              const vector<MultiResolutionHistogram*>& hists) {
   assert(output_stream.good());
   int32_t num_hists = hists.size();
   output_stream.write((char *)&num_hists, sizeof(int32_t));

   for (int ii = 0; ii < (int)hists.size(); ++ii) {
      WriteSingleHistogramToStream(output_stream, hists[ii]);
   }
}

}  // namespace libpmk
