// Copyright 2007, Massachusetts Institute of Technology.
// The use of this code is permitted for research only. There is
// absolutely no warranty for this software.
//
// Author: John Lee (jjl@mit.edu)
//

#ifndef HISTOGRAMS_MULTI_RESOLUTION_HISTOGRAM_H
#define HISTOGRAMS_MULTI_RESOLUTION_HISTOGRAM_H

#include <iostream>
#include "histograms/bin.h"

using namespace std;

namespace libpmk {
/// \brief A data structure for a pyramid of histograms, with a link
/// structure between levels.
/**
 *
 * A multi-resolution histogram is a tree-like arrangement of Bins.
 * This implementation allows one to quickly get the parent, child, or
 * siblings of any bin.
 *
 * There are two ways to access Bins in these data structures: either
 * by looking at the root bin and traversing the tree manually by
 * using the parent/child/sibling relationships, or by specifying a
 * LargeIndex describing a path down the tree. These indices are set
 * up and maintained such that at the nth level of the tree, the
 * LargeIndex has size n (the root node has size 0 which is an empty
 * LargeIndex).  The ith element of the LargeIndex specifies the link
 * traversed to get from the (i-1)st level to the ith level.
 *
 * The sibling pointers are managed such that GetNextSibling() always
 * returns a Bin with the next highest index in that branch of the
 * tree. This multi-resolution histogram is stored sparsely: if you
 * consider the Bins referred to by [0 1] and [0 3], it is possible
 * that [0 3] is the sibling after [0 1] (as long as there is no such
 * bin [0 2]).
 *
 * Note that using LargeIndex <i>and</i> the link structure between
 * Bins is redundant, as a LargeIndex specifies a path down the
 * tree. When reading and writing the multi-resolution histogram to
 * disk, we only persist the LargeIndexes-- the link structure is
 * inferred from the LargeIndex values upon reading from disk. The
 * purpose of the link structure is to speed up computations and
 * lookups.
 */

class MultiResolutionHistogram {
 public:
   MultiResolutionHistogram();
   ~MultiResolutionHistogram();

   int GetNumLevels() const;
   int GetNumBins() const;

   /// Get a pointer to the bin with the specified index.
   /**
    * Returns NULL if the bin is not found.
    */
   Bin* GetBin(LargeIndex index);

   /// Get a pointer to the bin with specified index
   /**
    * Same as GetBin(LargeIndex), but localizes the search to the
    * subtree given by finger. Returns NULL if there is no such Bin
    * in a subtree of <finger>.
    */
   Bin* GetBin(LargeIndex index, Bin* finger);

   /// Get a pointer to the root bin.
   Bin* GetRootBin();
   Bin* const GetRootBin() const;

   /// Insert a copy of the given bin into the tree.
   /**
    * Returns a pointer to the newly-added bin in the tree. This
    * function completely ignores any parent/child/sibling pointers in
    * <new_bin>.
    *
    * This function requires the parent bin to exist. It will not
    * create new bins along the way (it will abort if there is no
    * parent bin, i.e., a bin whose index is a prefix of that of
    * <new_bin> and whose index size is exactly 1 less than the new
    * bin's index size. The insertion happens such that the sibling
    * relationships remain sorted by index.
    *
    * If there is already is a bin with the given index,
    * we add the counts and keep the larger of the two sizes. This
    * applies to the root bin as well.
    *
    */
   Bin* AddBin(const Bin& new_bin);

   // Same as AddBin(), except if the user already has a pointer to the
   // parent, use that. If the parent is invalid, NULL is returned and 
   // no bin is added. In order for this to work properly, the parent pointer
   // MUST be a bin which can be accessed through child pointers of
   // root_bin_. In other words, this MultiResolutionHistogram must be the
   // owner of the parent that you pass into it. We do NOT check for this;
   // it is up to the client to make sure of it.
   /// Insert a copy of the given bin into the tree.
   /**
    * Same as AddBin(const Bin&), except it starts the search for the
    * bin at <finger>. A parent to <new_bin> must already exist, and
    * must be present in a sub-branch of <finger> (it may also be
    * <finger>).
    */ 
   Bin* AddBin(const Bin& new_bin, Bin* finger);

   /// Reads all the MultiResolutionHistograms in a stream.
   /**
    * File format: <ul>
    * <li> (int32) N, the number of MultiResolutionHistograms.
    * <li> (N * MultiResolutionHistogram) The histograms. </ul>
    *
    * Aborts if input_stream cannot be read.
    */
   static vector<MultiResolutionHistogram*>
      ReadFromStream(istream& input_stream);

   /// Reads some MultiResolutionHistograms from a stream.
   /**
    * <start> specifies the index of the first histogram to read in the
    * stream of data. If <selection_size> is large enough so that this
    * would want to read past the end of the file,
    * ReadSelectionFromStream will just stop reading. Observe that you
    * will need to adjust indices (i.e., if you choose to read Y
    * histograms starting from index X, then result[0] will be
    * the 6th histogram, as opposed to this[5]).
    *
    * Aborts if the input_stream cannot be read.
    */
   static vector<MultiResolutionHistogram*>
      ReadSelectionFromStream(istream& input_stream,
                              int start, int selection_size);

   /// Writes all of the MultiResolutionHistograms to a stream.
   /**
    * Aborts if the stream is bad. See ReadFromStream for the file
    * format.  \sa ReadFromStream
    */
   static void WriteToStream(ostream& output_stream,
                             const vector<MultiResolutionHistogram*>& hists);
   
   /// Reads just one histogram from a stream.
   /**
    * File format: <ul>
    * <li> (int32) The total number of bins, including the root
    * <li> For each bin: <ul>
    *     <li> (int32) L, the size of its index (0 for the root, etc.)
    *     <li> (L * int32) the index
    *     <li> (double) size
    *     <li> (double) count
    *
    * The ordering of the bins is a depth-first traversal of the tree.
    *
    * Aborts if the stream is bad.
    */
   static MultiResolutionHistogram*
      ReadSingleHistogramFromStream(istream& input_stream);

   /// Throws out one histogram's worth of data from the stream.
   static void IgnoreSingleHistogramFromStream(istream& input_stream);
   
   /// Write just one histogram to a stream.
   /**
    * Aborts if the stream is bad. See ReadSingleHistogramFromStream
    * for the format. \sa ReadSingleHistogramFromStream
    */
   static void WriteSingleHistogramToStream(ostream& output_stream,
                                            const MultiResolutionHistogram* h);
   
 private:
   // This deletes all bins, including the root bin. It then creates
   // a new empty root bin, making the resulting tree equivalent to
   // one that has just been initialized with the default constructor.
   void FreeAllBins();

   // Decrements num_bins_ automatically.
   void DeleteLeafBin(Bin* bin);

   int num_levels_;
   int num_bins_;
   Bin* root_bin_;
   
}; 
}  // namespace libpmk
#endif  // HISTOGRAMS_MULTI_RESOLUTION_HISTOGRAM_H
