// Copyright 2007, Massachusetts Institute of Technology.
// The use of this code is permitted for research only. There is
// absolutely no warranty for this software.
//
// Author: John Lee (jjl@mit.edu)
//
// Takes a point set list. For each point set, it keeps <percentage>%
// random points and removes the rest. It then writes the smaller
// point set list to the specified output file.

#include <assert.h>
#include <math.h>
#include <iostream>
#include <fstream>
#include "point_set/mutable-point-set-list.h"

using namespace std;
using namespace libpmk;

int main(int argc, char** argv) {
   if (argc != 4) {
      fprintf(stderr,
              "Usage: %s <input_point_set_list> <output_file> <percentage>\n",
              argv[0]);
      return 0;
   }

   string input_file(argv[1]);
   string output_file(argv[2]);
   double percentage = atof(argv[3]);
   assert(percentage <= 1);

   MutablePointSetList point_sets;
   ifstream input_stream(input_file.c_str(), ios::binary);
   point_sets.ReadFromStream(input_stream);
   input_stream.close();

   printf("Finished reading point sets.\n");


   for (int ii = 0; ii < point_sets.GetNumPointSets(); ++ii) {
      int num_points = point_sets[ii].GetNumFeatures();
      int num_to_pick = (int)ceil(percentage * num_points);

      // This bit array contains "true" for features we want to keep
      // and "false" for the rest.
      //printf("randomly sampling\n");
      vector<bool> include_set(num_points, false);
      int num_included = 0;
      while (num_included < num_to_pick) {
         int candidate = rand() % num_points;
         if (include_set.at(candidate) == false) {
            include_set.at(candidate) = true;
            num_included++;
         }
      }
      
      //printf("done sampling\n");
      for (int jj = num_points-1; jj >= 0; --jj) {
         if (include_set[jj] == false) {
            //printf("remove feature %d\n", jj);
            point_sets[ii].RemoveFeature(jj);
         }
      }
      printf("Finished sampling point set %d, %d from %d total.\n",
             ii, num_to_pick, num_points);
   }

   printf("Writing a total of %d points.\n", point_sets.GetNumPoints());
   ofstream output_stream(output_file.c_str(), ios::binary | ios::trunc);
   point_sets.WriteToStream(output_stream);
   output_stream.close();
/*
   // Build the new point set list, with the same structure as the old one.
   PointSetList new_list;
   int index = 0;
   for (int ii = 0; ii < point_sets.GetNumPointSets(); ++ii) {
      printf("Adding point set %d\n", ii);
      PointSet new_set(point_sets[ii].GetFeatureDim());
      for (int jj = 0; jj < point_sets[ii].GetNumFeatures(); ++jj) {
         if (include_set[index] == true) {
            new_set.AddFeature(point_sets[ii][jj]);
         }
         index++;
      }
      new_list.AddPointSet(new_set);
   }

   new_list.WritePointSetsToFile(output_file);*/


   // Delete points that we don't want. Notice that we go backwards,
   // since we don't want to screw up the indexing as we're doing this.
   
   return 0;

}
